#include "dumbres.hpp"
#include <math.h>

using namespace cg_engine;

Background::Background(int w, int h)
{
	int c = 0;
	double r;
	double x;
	double y;
	start = 0.0;
	end   = 130.0;

	backgroundshader = new ShaderProgram();
	backgroundshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	backgroundshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	backgroundshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);

	particles[c++] = new Particle(100, -3.0, -2.0, -3.0, -2.0, 22.0, 20.0); // left eye
	particles[c++] = new Particle(100,  3.0,  2.0, -3.0, -2.0, 22.0, 20.0); // right eye

	// smile
	for(r = 0.0; r < M_PI; r += (M_PI / 19.0)){
		x = cos(r) * 2.0;
		y = sin(r) * 2.0;
		particles[c] = new Particle(20, x - 0.1, x + 0.1, y - 0.1, y + 0.1, 22.0, 20.0);
		c++;
	}
	// face
	for(r = 0.0; r < 2.0 * M_PI; r += (2.0 * M_PI / 100.0)){
		x = cos(r) * 5.0;
		y = sin(r) * 5.0;
		particles[c] = new Particle(100, x - 0.2, x + 0.2,y - 0.2, y + 0.2, 22.0, 20.0);
		c++;
	}
}

void Background::show(double time)
{
	double eye[] = {0.0, 0.0, -2.0};
        double target[] = {0.0, 0.0, 0.0};
        double up[] = {0.0, -1.0, 0.0};

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	backgroundshader->use();
        Matrix camera;
        camera.lookAt(eye, target, up);
        Matrix tmp = camera * pmatrix;

        backgroundshader->setParam("pmatrix", tmp);
        backgroundshader->setParam("time", (float) time);

	for(int i = 0; i < 122; i++){
		particles[i]->update(time);
		particles[i]->draw();
	}
}

void Begin::createBegin()
{
	int chrBmask[] = {0, 1, 2, 3, 5, 9, 10, 14, 15, 16, 17, 18, 20, 24, 25, 29, 30, 31, 32, 33};
	int chrEmask[] = {0, 1, 2, 3, 4, 5, 10, 15, 16, 17, 18, 19, 20, 25, 30, 31, 32, 33, 34};
	int chrGmask[] = {1, 2, 3, 5, 9, 10, 15, 18, 19, 20, 24, 25, 29, 31, 32, 33};
	int chrImask[] = {1, 2, 3, 7, 12, 17, 22, 27, 31, 32, 33};
	int chrNmask[] = {0, 4, 5, 6, 9, 10, 12, 14, 15, 17, 19, 20, 22, 24, 25, 28, 29, 30, 34};

	Character *chrB = new Character(-0.30, -0.25, 0.0, 0.009, chrBmask, 20);
	Character *chrE = new Character(-0.25, -0.25, 0.0, 0.009, chrEmask, 19);
	Character *chrG = new Character(-0.20, -0.25, 0.0, 0.009, chrGmask, 16);
	Character *chrI = new Character(-0.15, -0.25, 0.0, 0.009, chrImask, 11);
	Character *chrN = new Character(-0.10, -0.25, 0.0, 0.009, chrNmask, 19);

	begin = new PString();
	begin->addChar(chrB);
	begin->addChar(chrE);
	begin->addChar(chrG);
	begin->addChar(chrI);
	begin->addChar(chrN);
}

Begin::Begin(int w, int h)
{
	start = 1.983;
	end   = 3.268;

	createBegin();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void Begin::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;
	rot.rotate(0.0, (time - start) / 4.0, 0.0);
	Matrix tmp = rot * camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	if(time > 3.0){
		begin->update(time - 3.0);
	}
	begin->draw();
}

void Fourteen::createFourteen(){
	int chr1mask[] = {2,3,6,8,10,13,18,23,28,33};
	int chr4mask[] = {1,6,11,16,18,21,22,23,24,28,33};

	Character *chr1 = new Character(0.30, 0.25, 0.0, 0.009, chr1mask, 10);
	Character *chr4 = new Character(0.35, 0.25, 0.0, 0.009, chr4mask, 11);

	fourteen = new PString();
	fourteen->addChar(chr1);
	fourteen->addChar(chr4);
}

Fourteen::Fourteen(int w, int h)
{
	start = 4.012;
	end   = 15.342;

	createFourteen();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void Fourteen::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;
	rot.rotate((time - start) / 5.0, 0.0, 0.0);
	Matrix tr;
	tr.translate(0.0 - (time - start) / 5.0, 0.0 + ((time - start) / 10.0), 0.0);
	Matrix tmp = tr * rot * camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	if(time > 4.9){
		fourteen->update(time);
	}
	fourteen->draw();
}

void Thirtytwo::createThirtytwo(){
	int chr3mask[] = {1,2,3,5,9,14,17,18,19,24,25,29,31,32,33};
	int chr2mask[] = {1,2,3,5,9,14,18,22,26,30,31,32,33,34};

	Character *chr3 = new Character(-0.30, 0.15, 0.0, 0.009, chr3mask, 15);
	Character *chr2 = new Character(-0.25, 0.15, 0.0, 0.009, chr2mask, 14);

	thirtytwo = new PString();
	thirtytwo->addChar(chr3);
	thirtytwo->addChar(chr2);
}

Thirtytwo::Thirtytwo(int w, int h)
{
	start = 6.018;
	end   = 130;//7.505;

	createThirtytwo();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void Thirtytwo::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;

	if(time > 9.0){
		thirtytwo->noreborn();
		thirtytwo->unify((time - 9.0) / 10.0, -1.0 * (time - 9.0) / 10.0, 0.0);
		thirtytwo->update(time);
		rot.rotate(0.0, (time - 9.0) / 4.0, 0.0);
	} else {
		rot.rotate(0.0, 0.0, 0.0);
	}

	Matrix tmp = rot * camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	thirtytwo->draw();
}

void N158::createN158(){
	int chr1mask[] = {2,3,6,8,10,13,18,23,28,33};
	int chr5mask[] = {0,1,2,3,4,5,10,15,16,17,18,24,29,30,31,32,33};
	int chr8mask[] = {1,2,3,5,9,10,14,16,17,18,20,24,25,29,31,32,33};

	Character *chr1 = new Character(-0.10, -0.05, 0.0, 0.009, chr1mask, 10);
	Character *chr5 = new Character(-0.05, -0.05, 0.0, 0.009, chr5mask, 17);
	Character *chr8 = new Character( 0.00, -0.05, 0.0, 0.009, chr8mask, 17);

	n158 = new PString();
	n158->addChar(chr1);
	n158->addChar(chr5);
	n158->addChar(chr8);
}

N158::N158(int w, int h)
{
	start = 8.474;
	end   = 9.939;

	createN158();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void N158::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix tr;
	tr.translate(0.0, 0.0, 0.0 - (time - start) * 1.4);
	Matrix tmp = tr * camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	if(time > 9.3){
		n158->update(time - start);
	}
	n158->draw();
}

void N271::createN271(){
	int chr2mask[] = {1,2,3,5,9,14,18,22,26,30,31,32,33,34};
	int chr7mask[] = {0,1,2,3,4,9,14,18,22,26,30};
	int chr1mask[] = {2,3,6,8,10,13,18,23,28,33};

	Character *chr2 = new Character(-0.20, -0.15, 0.0, 0.009, chr2mask, 14);
	Character *chr7 = new Character(-0.15, -0.15, 0.0, 0.009, chr7mask, 11);
	Character *chr1 = new Character(-0.10, -0.15, 0.0, 0.009, chr1mask, 10);

	n271 = new PString();
	n271->addChar(chr2);
	n271->addChar(chr7);
	n271->addChar(chr1);
}

N271::N271(int w, int h)
{
	start = 10.525;
	end   = 130;

	createN271();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void N271::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;
	rot.rotate((time - start) / 2.0, 0.0, 0.0);
	Matrix tmp = rot * camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	if(time > 12.0){
		n271->noreborn();
		n271->unify(0.02, 0.0, 0.0);
		n271->update(time - start);
	}
	n271->draw();
}

void Million::createMillion(){
	int chr1mask[] = {2,3,6,8,10,13,18,23,28,33};
	int chr0mask[] = {1,2,3,5,9,10,14,15,19,20,24,25,29,31,32,33};
	int chr6mask[] = {3,7,11,15,16,17,18,20,24,25,29,31,32,33};

	Character *chr1 = new Character(-0.30, -0.25, 0.0, 0.009, chr1mask, 10);
	Character *chr0 = new Character(-0.25, -0.25, 0.0, 0.009, chr0mask, 16);
	Character *chr6 = new Character(-0.20, -0.30, 0.0, 0.004, chr6mask, 14);

	million = new PString();
	million->addChar(chr1);
	million->addChar(chr0);
	million->addChar(chr6);
}

Million::Million(int w, int h)
{
	start = 12.486;
	end   = 130.0;

	createMillion();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void Million::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;
	rot.rotate(time / 5.0, time / 4.0, time / 3.0);
	Matrix tmp = /*rot **/ camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);
	if(time > 12.9){
		if(time > 15.0){
			million->unify(0.0, 0.1, 0.0);
		}
		million->noreborn();
		million->update(time - 12.9);
	}

	million->draw();
}

void Marioplex::createMarioplex(){
	int chr1mask[] = {2,3,6,8,10,13,18,23,28,33};
	int chr0mask[] = {1,2,3,5,9,10,14,15,19,20,24,25,29,31,32,33};
	int chr2mask[] = {1,2,3,5,9,14,18,22,26,30,31,32,33,34};
	int chr3mask[] = {1,2,3,5,9,14,17,18,19,24,25,29,31,32,33};
	int chr4mask[] = {1,6,11,16,18,21,22,23,24,28,33};

	Character *chr1b = new Character(-0.35,  0.25, 0.0, 0.02, chr1mask, 10);
	Character *chr0b = new Character(-0.25,  0.25, 0.0, 0.02, chr0mask, 16);
	Character *chr1s = new Character(-0.17,  0.20, 0.0, 0.008, chr1mask, 10);
	Character *chr2s = new Character(-0.13,  0.20, 0.0, 0.008, chr2mask, 14);
	Character *chr4s = new Character(-0.09,  0.20, 0.0, 0.008, chr4mask, 11);
	Character *chr3s = new Character(-0.05,  0.20, 0.0, 0.008, chr3mask, 15);
	Character *chr1m = new Character(-0.01,  0.20, 0.0, 0.008, chr1mask, 10);

	marioplex = new PString();
	marioplex->addChar(chr1b);
	marioplex->addChar(chr0b);
	marioplex->addChar(chr1s);
	marioplex->addChar(chr2s);
	marioplex->addChar(chr4s);
	marioplex->addChar(chr3s);
	marioplex->addChar(chr1m);
}

Marioplex::Marioplex(int w, int h)
{
	start = 14.289;
	end   = 17.551;

	createMarioplex();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void Marioplex::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix rot;
	rot.rotate(time / 5.0, time / 4.0, time / 3.0);
	Matrix tmp = /*rot **/ camera * pmatrix;

	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	marioplex->draw();
}

void RandomNums::createRandomNums(){
	int chrmask[10][35] = {
		{1,2,3,5,9,10,14,15,19,20,24,25,29,31,32,33},
		{2,3,6,8,10,13,18,23,28,33},
		{1,2,3,5,9,14,18,22,26,30,31,32,33,34},
		{1,2,3,5,9,14,17,18,19,24,25,29,31,32,33},
		{1,6,11,16,18,21,22,23,24,28,33},
		{0,1,2,3,4,5,10,15,16,17,18,24,29,30,31,32,33},
		{3,7,11,15,16,17,18,20,24,25,29,31,32,33},
		{0,1,2,3,4,9,14,18,22,26,30},
		{1,2,3,5,9,10,14,16,17,18,20,24,25,29,31,32,33},
		{1,2,3,5,9,10,14,16,17,18,19,23,27,31}
	};
	int masklen[] = {16,10,14,15,11,17,14,11,17,14};

	nums[0] = new Character(-0.30, -0.25, 0.0, 0.009, chrmask[0], 20);
	nums[1] = new Character(-0.25, -0.25, 0.0, 0.009, chrmask[1], 19);
	nums[2] = new Character(-0.20, -0.25, 0.0, 0.009, chrmask[2], 16);
	nums[3] = new Character(-0.15, -0.25, 0.0, 0.009, chrmask[3], 11);
	nums[4] = new Character(-0.10, -0.25, 0.0, 0.009, chrmask[4], 19);
	nums[5] = new Character(-0.30, -0.25, 0.0, 0.009, chrmask[5], 20);
	nums[6] = new Character(-0.25, -0.25, 0.0, 0.009, chrmask[6], 19);
	nums[7] = new Character(-0.20, -0.25, 0.0, 0.009, chrmask[7], 16);
	nums[8] = new Character(-0.15, -0.25, 0.0, 0.009, chrmask[8], 11);
	nums[9] = new Character(-0.10, -0.25, 0.0, 0.009, chrmask[9], 19);

	rndnum = new PString();
	GLfloat xpos = -0.30;
	for(int i = 0; i < 100; i++){
		int rndchr = trunc(drand48() * 10.0);
		rndnum->addChar(new Character(xpos, -0.25, 0.0, 0.05, chrmask[rndchr], masklen[rndchr]));
		xpos = xpos + 0.3;
	}
}

RandomNums::RandomNums(int w, int h)
{
	start = 18.0;
	end   = 130;

	createRandomNums();

	starshader = new ShaderProgram();
	starshader->addShader("data/stars.vs", GL_VERTEX_SHADER);
	starshader->addShader("data/stars.fs", GL_FRAGMENT_SHADER);
	starshader->compile();
	pmatrix.perspective(35.0, w / h, 0.1, 1500);
}

void RandomNums::show(double time)
{

	starshader->use();

	double eye[] = {0.0, 0.0, -2.0};
	double target[] = {0.0, 0.0, 0.0};
	double up[] = {0.0, -1.0, 0.0};
	Matrix camera;
	camera.lookAt(eye, target, up);
	Matrix tr;
	if(time < 29.0){
		tr.translate((time - start) * -3.0, 0.0, 0.0);
	} else {
		tr.translate((time - 29.0) * -5.0, 0.0, 0.0);
	}
	Matrix tmp = tr * camera * pmatrix;
	starshader->setParam("pmatrix", tmp);
	starshader->setParam("time", (float) time);

	rndnum->draw();
}

MyDemo::MyDemo(int w, int h): Demo(w, h){
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_PROGRAM_POINT_SIZE);
}

void MyDemo::init(){
	scenes.push_back(new Background(screenwidth, screenheight));
	scenes.push_back(new Begin(screenwidth, screenheight));
	scenes.push_back(new Fourteen(screenwidth, screenheight));
	scenes.push_back(new Thirtytwo(screenwidth, screenheight));
	scenes.push_back(new N158(screenwidth, screenheight));
	scenes.push_back(new N271(screenwidth, screenheight));
	scenes.push_back(new Million(screenwidth, screenheight));
	scenes.push_back(new Marioplex(screenwidth, screenheight));
	scenes.push_back(new RandomNums(screenwidth, screenheight));
}

